# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Allen Lowe <lallenlowe@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
import pango
import warnings

from dexter.helpers import get_builder
import dexter.backend.utilities as utilities

import gettext
from gettext import gettext as _
gettext.textdomain('dexter')

class HintedEntry(gtk.Entry):
    """a gtk.Entry with a DanRabbit style hint built in"""
    __gtype_name__ = 'HintedEntry'

    def __init__(self, hint_string):
        """create a new HintedEntry
        
        hint_string is the string to display as a grey, italic hint
        """
        gtk.Entry.__init__(self)
        self.hint_string = hint_string
        self.hint()
        self.connect("focus-in-event", self.focused)
        self.connect("focus-out-event", self.focus_out)

    def focused(self, widget, event):
        """this HintedEntry gets focused"""
        if not self.get_text():
            self.unhint()

    def focus_out(self, widget, event):
        """focus leaves this HintedEntry"""
        if not self.get_text():
            self.hint()

    def hint(self):
        """activate the grey, italic hint"""
        self.set_text(self.hint_string)
        self.grey_out()

    def unhint(self, string=""):
        """remove the hint and reset the font"""
        self.set_text(string)
        self.reset_font()
        
    def is_hinted(self):
        """check if this entry contains it's hint_string"""
        if not self.get_text():
            return True
        else:
            return False

    def grey_out(self):
        """set the font to Italic and grey, for hinting"""
        self.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse("#999"))
        self.modify_font(pango.FontDescription("italic"))

    def reset_font(self):
        """reset the text color to normal so the user can edit"""
        #FIXME: this is not the EXACT right color for the default font...
        self.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse("#444"))
        self.modify_font(pango.FontDescription("normal"))

    def get_text(self):
        text = super(HintedEntry, self).get_text()
        # If all that is left is the hint string, return nothing
        if text == self.hint_string:
            return ""
        else:
            return text


class SearchBar(HintedEntry):
    """A HintedEntry with the search logic built in"""
    __gtype_name__ = "SearchBar"

    def __init__(self, hint_string, dbus):
        """create a new SearchBar, data is the list of contacts to search"""
        HintedEntry.__init__(self, hint_string)
        self.dbus = dbus
        self.SearchContacts = self.dbus.get_dbus_method("SearchContacts")
        self.searching = False
        self.previous_search = ""
        self.hint()
        self.result_set = []
        self.result_sets = {}
        self.set_icon_from_icon_name('primary', 'gtk-find')
        self.connect("icon-press", self.icon_pressed)
        self.agr = gtk.AccelGroup()
        key, mod = gtk.accelerator_parse("<Control>f")
        self.add_accelerator("grab-focus", self.agr, key, mod, gtk.ACCEL_LOCKED)
        self.setup_clear_icon()

    def setup_clear_icon(self):
        stock_item = ('edit-clear-symbolic', None, 0, 0, gtk.STOCK_CLEAR)
        factory = gtk.IconFactory()
        icon_set = gtk.IconSet()
        icon_source = gtk.IconSource()
        icon_source.set_icon_name(gtk.STOCK_CLEAR)
        icon_set.add_source(icon_source)
        icon_source = gtk.IconSource()
        icon_source.set_icon_name('edit-clear-symbolic')
        icon_set.add_source(icon_source)
        factory.add('edit-clear-symbolic', icon_set)
        gtk.stock_add([stock_item])
        factory.add_default()
        

    def refresh(self, num_contacts):
        """refresh the contact list to search
        
        make SearchBar insensitive if data is empty
        """
        if num_contacts:
            self.set_sensitive(True)
        else:
            self.set_sensitive(False)

    def icon_pressed(self, widget, icon_pos, event):
        """the user clicks the search or clear button"""
        if icon_pos == gtk.ENTRY_ICON_SECONDARY:
            if not self.is_focus():
                self.searching = False
                self.hint()
            else:
                self.searching = False
                self.set_text("")
                self.set_icon_from_stock('secondary', None)
                self.searching = True

    def search(self):
        """live search 'data' with the text in the searchbar"""
        text = self.get_text()
        if self.searching:
            if not text:  # most likely text has just been cleared, or backspaced away
                #the SearchBar is empty so hide the clear icon
                self.set_icon_from_stock('secondary', None)
                #empty out the cached result sets
                self.result_sets = {}
                self.result_set = []
            else:
                #the SearchBar contains text, so show the clear button
                with warnings.catch_warnings():
                    warnings.simplefilter("ignore")
                    self.set_icon_from_stock('secondary', 'edit-clear-symbolic')
                if text in self.result_sets:
                    #if this string was already searched, use the same results again, used for fast backspacing
                    self.result_set = self.result_sets[text]
                else:  # else if the string hasn't been searched yet, or the result sets cache has been emptied
                    # if there is no result set already, search the entire contact list
                    self.result_set = self.SearchContacts(text)
                    # cache the result set for later use
                    self.result_sets[text] = self.result_set
                self.previous_search = text
                    
    def hint(self):
        """reset SearchBar to it's hinted state"""
        self.searching = False
        self.set_icon_from_stock('secondary', None)
        HintedEntry.hint(self)

    def focused(self, widget, event):
        if not self.searching:
            self.unhint()
            self.searching = True

    def focus_out(self, widget, event):
        if not self.get_text():
            self.searching = False
            self.hint()
